@@ -2,7 +2,7 @@ use crate::editorgui::{Packages, EditorConfig, ProjectConfig};
 use keeshond::gameloop::GameControl;
 use std::any::Any;
 use keeshond::renderer::{DrawControl, DrawTransform};
-use keeshond_datapack::DataMultistore;
+use keeshond::datapack::DataMultistore;
 
 pub mod sheetdocument;
 pub mod costumedocument;
@@ -12,7 +12,8 @@ pub mod leveldocument;
 
 pub struct DocumentState
 {
-    pub active : bool
+    pub active : bool,
+    pub make_active : bool
 }
 
 impl DocumentState
@@ -21,7 +22,8 @@ impl DocumentState
     {
         DocumentState
         {
-            active : false
+            active : false,
+            make_active : false
         }
     }
 }
