@@ -1,7 +1,7 @@
 use std::collections::{BTreeMap, BTreeSet, HashMap};
 use std::path::{Component, PathBuf};
 use std::sync::mpsc::{channel, Receiver};
-use std::time::Duration;
+use std::time::{Duration, Instant};
 
 use chrono::{DateTime, TimeZone, Utc};
 use notify::{DebouncedEvent, RecommendedWatcher, RecursiveMode, Watcher};
@@ -11,15 +11,14 @@ use walkdir::WalkDir;
 
 use keeshond::gameloop::GameControl;
 use keeshond::renderer::Sheet;
-use keeshond_datapack::{DataHandle, DataObject};
+use keeshond::datapack::{DataHandle, DataObject};
 use keeshond_treats::spawnable::{Representation, Spawnable, SpawnableSet};
-use keeshond_treats::visual::{Animation, AnimationCel, Costume, SliceHandle};
+use keeshond_treats::visual::{Animation, AnimationCel, AnimationFrame, AnimationTrack, Costume, SliceHandle};
 
 use crate::editorgui::Packages;
-use crate::import::importer::ase::{AsepriteBackgroundImporter, AsepriteSpriteImporter};
+use crate::importer::ase::{AsepriteBackgroundImporter, AsepriteSpriteImporter};
 use crate::import::sheet::{BackgroundImportData, ImportedImage, SpriteImportData};
 
-pub mod importer;
 pub mod sheet;
 
 const REGISTRY_CONFIG_FILENAME : &str = "Registry.toml";
@@ -604,6 +603,8 @@ impl AutoImport
 
         info!("Importing item \"{}\"", registry_path);
 
+        let import_start = Instant::now();
+
         let entry = self.registry.entry(registry_path.clone()).or_insert(RegistryEntry::default());
 
         if let Some(importer_name) = &entry.importer
@@ -648,6 +649,13 @@ impl AutoImport
         {
             self.update_registry_entry(pathbuf, success);
         }
+
+        if success || deferred
+        {
+            let import_time = (Instant::now() - import_start).as_millis();
+
+            info!("Processed \"{}\" in {} ms", registry_path, import_time);
+        }
     }
 
     fn do_import_shared_sheet(&mut self, package : &str, name : &str, group : &Vec<ImportGroupItem<SpriteImportData>>,
@@ -667,6 +675,12 @@ impl AutoImport
             let mut new_spawnables = AutoImport::read_existing_resource(packages, game,
                                                                         &item.data.dest_spawnable_package,
                                                                         &item.data.dest_spawnable).unwrap_or(SpawnableSet::new());
+            let mut default_track_name = item.data.default_track_name.clone();
+
+            if default_track_name.is_empty()
+            {
+                default_track_name = String::from("default");
+            }
 
             for import_animation in &item.data.animations
             {
@@ -674,24 +688,50 @@ impl AutoImport
 
                 for frame in &import_animation.frames
                 {
-                    animation.add_cel(AnimationCel
+                    animation.add_frame(AnimationFrame
                     {
-                        sheet : DataHandle::with_path(&package, &name),
-                        slice : SliceHandle::new(&format!("{}{}", prefix, frame.index)),
-                        delay : frame.delay,
-                        x : 0.0,
-                        y : 0.0
-                    })
+                        delay : frame.delay
+                    });
+
+                    for track in &frame.tracks
+                    {
+                        let mut track_name = track.track_name.clone();
+
+                        if track_name.is_empty()
+                        {
+                            track_name = default_track_name.clone();
+                        }
+
+                        let costume_track_id = match new_costume.track_id(&track_name)
+                        {
+                            None => { new_costume.add_track(&track_name) }
+                            Some(id) => { id }
+                        };
+
+                        let row = match animation.costume_track_to_row(costume_track_id as u16)
+                        {
+                            None => { animation.add_track(AnimationTrack::new(costume_track_id as u16)) }
+                            Some(row) => { row }
+                        };
+
+                        animation.set_cel(animation.frame_count() - 1, row, AnimationCel
+                        {
+                            sheet : DataHandle::with_path(&package, &name),
+                            slice : SliceHandle::new(&format!("{}{}", prefix, track.index)),
+                            x : 0.0,
+                            y : 0.0
+                        });
+                    }
                 }
 
                 animation.set_looping(import_animation.looping);
 
-                match new_costume.animation_id(&import_animation.name)
+                match new_costume.animation_id(&import_animation.animation_name)
                 {
                     None =>
                     {
                         new_costume.add_animation(animation);
-                        new_costume.rename_animation(new_costume.animation_count() - 1, &import_animation.name);
+                        new_costume.rename_animation(new_costume.animation_count() - 1, &import_animation.animation_name);
                     }
                     Some(id) => { new_costume.set_animation(id, animation); }
                 }
@@ -710,10 +750,17 @@ impl AutoImport
                 new_spawnables.rename_spawnable(new_spawnables.spawnable_count() - 1, &item.data.spawn_type_name);
             }
 
-            AutoImport::write_imported_resource(packages, game, &item.data.dest_costume_package,
-                                                &item.data.dest_costume, &mut new_costume)?;
-            AutoImport::write_imported_resource(packages, game, &item.data.dest_spawnable_package,
-                                                &item.data.dest_spawnable, &mut new_spawnables)?;
+            if !item.data.dest_costume_package.is_empty() && !item.data.dest_costume.is_empty()
+            {
+                AutoImport::write_imported_resource(packages, game, &item.data.dest_costume_package,
+                                                    &item.data.dest_costume, &mut new_costume)?;
+
+                if !item.data.dest_spawnable_package.is_empty() && !item.data.dest_spawnable.is_empty()
+                {
+                    AutoImport::write_imported_resource(packages, game, &item.data.dest_spawnable_package,
+                                                        &item.data.dest_spawnable, &mut new_spawnables)?;
+                }
+            }
         }
 
         AutoImport::write_imported_resource(packages, game, package, name, &mut new_sheet)?;
@@ -728,6 +775,8 @@ impl AutoImport
 
         info!("Handling shared sheet for \"{}/{}\" with {} new or changed source item(s)", package, name, group.len());
 
+        let import_start = Instant::now();
+
         match self.do_import_shared_sheet(package, name, group, packages, game)
         {
 
@@ -742,6 +791,13 @@ impl AutoImport
         {
             self.update_registry_entry(&item.pathbuf, success);
         }
+
+        if success
+        {
+            let import_time = (Instant::now() - import_start).as_millis();
+
+            info!("Updated shared sheet \"{}/{}\" in {} ms", package, name, import_time);
+        }
     }
 
     pub fn import_pending(&mut self, packages : &mut Packages, game : &mut GameControl) -> ImportPendingResult
@@ -828,7 +884,7 @@ impl AutoImport
 
             ui.spacing();
 
-            crate::util::window_placeholder_text(&format!("Items needing configuration ({})", self.needs_config_files.len()), ui);
+            crate::util::widget::window_placeholder_text(&format!("Items needing configuration ({})", self.needs_config_files.len()), ui);
 
             ui.spacing();
 
@@ -852,7 +908,7 @@ impl AutoImport
                 }
             }
 
-            crate::util::window_placeholder_text(&format!("Configured items ({})", self.configured_files.len()), ui);
+            crate::util::widget::window_placeholder_text(&format!("Configured items ({})", self.configured_files.len()), ui);
 
             ui.spacing();
 
@@ -912,7 +968,7 @@ impl AutoImport
 
         ui.spacing();
         ui.spacing();
-        crate::util::column_separator(ui);
+        crate::util::widget::column_separator(ui);
         ui.spacing();
         ui.spacing();
 
@@ -987,7 +1043,7 @@ impl AutoImport
 
         if !any_importers
         {
-            crate::util::window_placeholder_text("(No importers available for this filetype)", ui);
+            crate::util::widget::window_placeholder_text("(No importers available for this filetype)", ui);
         }
 
         choice
@@ -1011,7 +1067,7 @@ impl AutoImport
         {
             None =>
             {
-                crate::util::window_placeholder_text("(This importer is not available)", ui);
+                crate::util::widget::window_placeholder_text("(This importer is not available)", ui);
             }
             Some(importer) =>
             {
@@ -1020,7 +1076,7 @@ impl AutoImport
                     entry.config = Some(ImportConfig::new());
                 }
 
-                crate::util::column_separator(ui);
+                crate::util::widget::column_separator(ui);
 
                 ui.spacing();
                 ui.spacing();
@@ -1068,8 +1124,8 @@ impl AutoImport
         let size = ui.io().display_size;
 
         if let Some(import_window) = imgui::Window::new("AutoImport")
-            .position([size[0] / 2.0, size[1] / 2.05], imgui::Condition::Appearing)
-            .position_pivot([0.5, 0.50]).size([850.0, 550.0], imgui::Condition::Appearing)
+            .position([size[0] / 2.0, size[1] / 1.98], imgui::Condition::Appearing)
+            .position_pivot([0.5, 0.50]).size([850.0, 600.0], imgui::Condition::Appearing)
             .collapsible(false).opened(open).save_settings(false).begin(ui)
         {
             ui.columns(2, "##auto_import_columns", false);
@@ -1083,7 +1139,7 @@ impl AutoImport
             {
                 None =>
                 {
-                    crate::util::window_placeholder_text("General Information", ui);
+                    crate::util::widget::window_placeholder_text("General Information", ui);
 
                     ui.spacing();
                     ui.spacing();
@@ -1092,7 +1148,7 @@ impl AutoImport
                 }
                 Some(item) =>
                 {
-                    crate::util::window_placeholder_text(&item.name, ui);
+                    crate::util::widget::window_placeholder_text(&item.name, ui);
 
                     ui.spacing();
                     ui.spacing();
