@@ -1,15 +1,14 @@
 #![windows_subsystem = "windows"]
 
 extern crate keeshond;
-extern crate keeshond_datapack;
 extern crate num;
 extern crate confy;
 extern crate image;
 extern crate palette;
 extern crate toodee;
-#[macro_use] extern crate try_or;
 #[macro_use] extern crate serde_derive;
 #[macro_use] extern crate log;
+extern crate sdl2;
 extern crate notify;
 extern crate walkdir;
 extern crate path_slash;
@@ -17,7 +16,7 @@ extern crate chrono;
 extern crate rectangle_pack;
 extern crate asefile;
 
-use keeshond::gameloop::{Gameloop, GameInfo};
+use keeshond::gameloop::{GameInfo, Gameloop};
 use keeshond::renderer::ViewportMode;
 use crate::editorgui::EditorGui;
 
@@ -26,6 +25,7 @@ mod editorgui;
 mod document;
 mod wizard;
 mod import;
+mod importer;
 
 ////////////////////////////////////////
 
@@ -38,7 +38,7 @@ fn main()
         friendly_name : "Keeshond Game Editor",
         base_width : 1280,
         base_height : 720,
-        viewport_mode : ViewportMode::OneToOne,
+        viewport_mode : ViewportMode::OneToOneUnscaled,
         texture_filtering : false,
         allow_system_sleep : true,
         low_energy_app : true,
